/*
 * Copyright (C) 2007 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#if !defined HATP_PARTIAL_PLAN_APPRAISER_HH_
#define HATP_PARTIAL_PLAN_APPRAISER_HH_

#include<string>
#include<vector>
#include<map>

class TaskInstance;
class TaskList;
class PlanTree;
class LightWorldBase;
class SocialRuleBase;

/** Structure to store the cost of the current plan and computes the penalties.
 * The penalties are computed from the social rules.
 * TODO This class in not complete, some part of the code is still missing.
 */
class PartialPlanAppraiser
{
private:
	/** Current projection cost, the sum of all actions cost
	*/
	float current_projection_cost;

	/** Current penalties, holds the sum of all the penalties (badDecomposition, ...)
	*/
	float current_penalties;

	/** The bad-decomposition tasks and the tasks they concern
	*/
	std::map<std::string, std::vector<std::string> > badDecompoTaskNames;

	/** The bad-decomposition tasks and the fact that their subtasks triggered (arranged in accord with badDecompoTaskNames)
	*/
	std::map<std::string, std::vector<bool> > badDecompoTaskNamesDetected;

	/** The bad-decomposition tasks associated to the current value of the penalty
	*/
	std::map<std::string,float> badDecompoPenalties;

	/** The undesirable-sequence tasks and the tasks they concern
	*/
	std::map<std::string, std::vector<std::string> > undSeqTaskNames;

	/** The undesirable-sequence tasks and the fact that their subtasks triggered (arranged in accord with badDecompoTaskNames)
	*/
	std::map<std::string, std::vector<bool> > undSeqTaskNamesDetected;

	/** The undesirable-sequence tasks associated to the current value of the penalty
	*/
	std::map<std::string,float> undSeqPenalties;

	/** The penalty associated to intricacy (how much two different streams of actions are tighted with causal links)
	*/
	float intricacyControlPenalty;

public:
	explicit PartialPlanAppraiser();
	PartialPlanAppraiser(PartialPlanAppraiser const& ppa);
	PartialPlanAppraiser& operator=(PartialPlanAppraiser const& other);
	void init();

	void addBadDecomposition(std::string const& name, std::vector<std::string> const& badDecompoSubtasks);
	void addUndesirableSequence(std::string const& name, std::vector<std::string> const& undSeqTasks);

	void addToCurrentProjectionCost(float toAdd);
	float getCurrentProjectionCost() const;

	std::map<std::string, std::vector<std::string> > const& getBadDecompoTaskNames() const;
	std::map<std::string, std::vector<bool> > const& getBadDecompoTaskNamesDetected() const;
	std::map<std::string, std::vector<std::string> > const& getUndSeqTaskNames() const;
	std::map<std::string, std::vector<bool> > const& getUndSeqTaskNamesDetected() const;
//	unsigned int getIntricacyControlNbLinks() const;

	float getCurrentPenalties() const;
	std::map<std::string,float> const& getBadDecompoPenalties() const;
	std::map<std::string,float> const& getUndSeqPenalties() const;
	float getIntricacyControlPenalty() const;

	void updateBadDecompoFields(SocialRuleBase const& srb, LightWorldBase const& wb, PlanTree const& tree);
	void updateBadDecompoFields(TaskList const& TL, SocialRuleBase const& srb, LightWorldBase const& wb, PlanTree const& tree);
//	void updateUndSeqFields(TaskInstance const& ti, SocialRuleBase const& srb, LightWorldBase const& wb, TaskList const& proj);
	void updateIntricacyControlFields(SocialRuleBase const& srb, TaskList const& proj);

	~PartialPlanAppraiser();

private:
	void updateBadDecompoFields(const std::set<std::string>& tasks, SocialRuleBase const& srb, LightWorldBase const& wb, PlanTree const& tree);
};

/** Default constructor: set all fields to 0.
 */
inline PartialPlanAppraiser::PartialPlanAppraiser()
{
	current_projection_cost=0.;
	current_penalties=0.;
	intricacyControlPenalty=0.;
}

/** Copy constructor.
 * @param ppa PartialPlanAppraiser to copy
 */
inline PartialPlanAppraiser::PartialPlanAppraiser(PartialPlanAppraiser const& ppa)
{
	current_projection_cost = ppa.getCurrentProjectionCost();
	current_penalties = ppa.getCurrentPenalties();
	badDecompoTaskNames = ppa.getBadDecompoTaskNames();
	undSeqTaskNames = ppa.getUndSeqTaskNames();
	badDecompoTaskNamesDetected = ppa.getBadDecompoTaskNamesDetected();
	undSeqTaskNamesDetected = ppa.getUndSeqTaskNamesDetected();
	badDecompoPenalties = ppa.getBadDecompoPenalties();
	undSeqPenalties = ppa.getUndSeqPenalties();
	intricacyControlPenalty = ppa.getIntricacyControlPenalty();
}

/** Get the current cost for the plan.
 * @return The current_projection_cost
 */
inline float PartialPlanAppraiser::getCurrentProjectionCost() const
{
	return current_projection_cost;
}

/** Get the sum of all the penalties on this plan.
 * @return The current_penalties
 */
inline float PartialPlanAppraiser::getCurrentPenalties() const
{
	return current_penalties;
}

/** Get the bad-decomposition tasks and the corresponding tasks.
 * @return The badDecompoTaskNames
 */
inline std::map<std::string, std::vector<std::string> > const& PartialPlanAppraiser::getBadDecompoTaskNames() const
{
	return badDecompoTaskNames;
}

/** Get the undesirable-sequence tasks and the corresponding tasks.
 * @return The undSeqTaskNames
 */
inline std::map<std::string, std::vector<std::string> > const& PartialPlanAppraiser::getUndSeqTaskNames() const
{
	return undSeqTaskNames;
}

/** Get the bad-decomposition tasks and whether their subtasks triggered or not.
 * @return The badDecompoTaskNamesDetected
 */
inline std::map<std::string, std::vector<bool> > const& PartialPlanAppraiser::getBadDecompoTaskNamesDetected() const
{
	return badDecompoTaskNamesDetected;
}

/** Get the undesirable-sequence tasks and whether their subtasks triggered or not.
 * @return The undSeqTaskNamesDetected
 */
inline std::map<std::string, std::vector<bool> > const& PartialPlanAppraiser::getUndSeqTaskNamesDetected() const
{
	return undSeqTaskNamesDetected;
}

/** Get the bad-decomposition tasks and the corresponding penalties.
 * @return The badDecompoPenalties
 */
inline std::map<std::string,float> const& PartialPlanAppraiser::getBadDecompoPenalties() const
{
	return badDecompoPenalties;
}

/** Get the undesirable-sequence tasks and the corresponding penalties.
 * @return The undSeqPenalties
 */
inline std::map<std::string,float> const& PartialPlanAppraiser::getUndSeqPenalties() const
{
	return undSeqPenalties;
}

/** Get the penalty due to the intricacy of the plans.
 * @return The intricacyControlPenalty
 */
inline float PartialPlanAppraiser::getIntricacyControlPenalty() const
{
	return intricacyControlPenalty;
}

/** Update the current cost for the plan.
 * This is called when an action is projected (see hatpPlanningSession::runPhase2()).
 * @param toAdd Value to add to current_projection_cost
 */
inline void PartialPlanAppraiser::addToCurrentProjectionCost(float toAdd)
{
	current_projection_cost += toAdd;
}

/** Destructor: empty.
 */
inline PartialPlanAppraiser::~PartialPlanAppraiser()
{}

#endif
